@@ -11,6 +11,14 @@
 		suite_add_tcase (suite, t); \
 	} while (0)
 
+#define TEST_CASE_WITH_FIXTURE(suite, test, name, setup, teardown) \
+	do { \
+		TCase *t = tcase_create (#name); \
+		tcase_add_checked_fixture (t, setup, teardown); \
+		tcase_add_test (t, test_##test##_##name); \
+		suite_add_tcase (suite, t); \
+	} while (0)
+
 #define MAIN(test) \
 	int main (int argc PIPELINE_ATTR_UNUSED, \
 		  char **argv PIPELINE_ATTR_UNUSED) \
@@ -25,3 +33,7 @@
 		return (failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE; \
 	}
 
+extern char *temp_dir;
+
+void temp_dir_setup (void);
+void temp_dir_teardown (void);
